Package: microbenchmark
Title: Accurate Timing Functions
Description: Provides infrastructure to accurately measure and compare
        the execution time of R expressions.
Authors@R: c(person("Olaf", "Mersmann", role=c("aut")),
        person("Claudia", "Beleites", role=c("ctb")),
        person("Rainer", "Hurling", role=c("ctb")),
        person("Ari", "Friedman", role=c("ctb")),
        person(given=c("Joshua","M."), family="Ulrich", role="cre",
               email="josh.m.ulrich@gmail.com"))
URL: https://github.com/joshuaulrich/microbenchmark/
BugReports: https://github.com/joshuaulrich/microbenchmark/issues/
License: BSD_2_clause + file LICENSE
Depends: R (>= 3.2.0)
Imports: graphics, stats
Suggests: ggplot2, multcomp, RUnit
SystemRequirements: On a Unix-alike, one of the C functions
        mach_absolute_time (macOS), clock_gettime or gethrtime. If none
        of these is found, the obsolescent POSIX function gettimeofday
        will be tried.
ByteCompile: yes
Version: 1.5.0
NeedsCompilation: yes
Packaged: 2024-09-03 22:50:33 UTC; josh
Author: Olaf Mersmann [aut],
  Claudia Beleites [ctb],
  Rainer Hurling [ctb],
  Ari Friedman [ctb],
  Joshua M. Ulrich [cre]
Maintainer: Joshua M. Ulrich <josh.m.ulrich@gmail.com>
Repository: CRAN
Date/Publication: 2024-09-04 18:30:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 08:52:13 UTC; unix
Archs: microbenchmark.so.dSYM

S3method(boxplot,microbenchmark)
S3method(print,microbenchmark)
S3method(rbind,microbenchmark)
S3method(summary,microbenchmark)

if (getRversion() >= "3.6.0") {
  S3method(ggplot2::autoplot, microbenchmark)
}

export(get_nanotime)
export(microbenchmark)
export(microtiming_precision)

importFrom(graphics,boxplot)
importFrom(stats,aggregate)
importFrom(stats,fivenum)
importFrom(stats,lm)

useDynLib(microbenchmark,do_get_nanotime)
useDynLib(microbenchmark,do_microtiming)
useDynLib(microbenchmark,do_microtiming_precision)

library(microbenchmark)

test.unit_is_object_errors <- function()
{
  out <- try(microbenchmark(NULL, unit=a), silent = TRUE)
  checkTrue(inherits(out, "try-error"))
}

test.unit_f_is_valid <- function()
{
  out <- try(microbenchmark(NULL, unit="f"), silent = TRUE)
  checkTrue(!inherits(out, "try-error"))
}

test.unit_is_int_errors <- function()
{
  out <- try(microbenchmark(NULL, unit=4), silent = TRUE)
  checkTrue(inherits(out, "try-error"))
}

kest.unit_arg_errors_before_printing <- function()
{
  out <- try(microbenchmark(NULL, unit="foo"), silent = TRUE)
  checkTrue(inherits(out, "try-error"))
}

test.unit_arg_valid_values <- function()
{
  check <- function(x, u)
  {
    unit <- microbenchmark:::determine_unit
    checkIdentical(unit(unit = u), attr(x, "unit"))
  }

  test <- function() {}

  values <- c("nanoseconds", "ns",
              "microseconds", "us",
              "milliseconds", "ms",
              "seconds", "s", "secs",
              "time", "t",
              "frequency", "f",
              "hz", "khz", "mhz",
              "eps", "relative")

  for (u in values) {
    out <- microbenchmark(test(), unit = u, times = 1)
    check(out, u)
  }
}

test.unit_is_null_does_not_error <- function()
{
  out <- try(print(microbenchmark(NULL, unit = NULL)), silent = TRUE)
  checkTrue(!inherits(out, "try-error"))
}

test.simple_timing <- function()
{
  set.seed(21)
  out <- microbenchmark(rnorm(1e4))
  checkTrue(all(out$time > 0))
}

test.get_nanotime <- function()
{
  nt <- get_nanotime()
  checkTrue(nt > 0)
}

test.microtiming_precision <- function()
{
  mtp <- tryCatch(microtiming_precision(),
                  warning = function(w) w,
                  error = function(e) e)

  if (is(mtp, "warning") || is(mtp, "error")) {
    stop(mtp$message)
  }
}

test.setup_expression <- function()
{
  set.seed(21)
  x <- rnorm(10)
  microbenchmark(y <- rnorm(10), x, setup = set.seed(21))
  checkTrue(identical(x, y))
}

test.setup_expression_check <- function()
{
  my_check <- function(values) {
    v1 <- values[[1]]
    all(sapply(values[-1], function(x) identical(v1, x)))
  }
  set.seed(21)
  x <- rnorm(10)
  microbenchmark(rnorm(10), x, check = my_check, setup = set.seed(21))
}

test.setup_expression_eval_env_check <- function()
{
  my_check <- function(values) {
    v1 <- values[[1]]
    all(sapply(values[-1], function(x) identical(v1, x)))
  }
  set.seed(21)
  x <- rnorm(10)
  microbenchmark(rnorm(n), x, check = my_check,
                 setup = {set.seed(21); n <- 10})
}

test.setup_expression_eval_env <- function()
{
  x <- rnorm(10)
  microbenchmark(y <- rnorm(n), x, setup = {n <- 10})
  checkTrue(length(y) == 10L)
}

test.expression_eval_parent_frame <- function()
{
  fx <- function() { 1:10 }
  fy <- function() { 1:10 }
  microbenchmark(x <- fx(), y <- fy())
  checkTrue(identical(x, y))
}

test.setup_expression_check_equal <- function()
{
  set.seed(21)
  x <- rnorm(1e5)
  microbenchmark(rnorm(1e5), x, check = 'equal', setup = set.seed(21))
}

test.setup_expression_check_equal_failure <- function()
{
  set.seed(21)
  x <- rnorm(1e5)
  attr(x = x, 'abc') <- 123 # add attribute
  out <- try(microbenchmark(rnorm(1e5), x, check = 'equal', setup = set.seed(21)), silent = T)
  checkTrue(inherits(out, "try-error"))
}

test.setup_expression_check_equivalent <- function()
{
  set.seed(21)
  x <- rnorm(1e5)
  attr(x = x, 'abc') <- 123 # add attribute
  microbenchmark(rnorm(1e5), x, check = 'equivalent', setup = set.seed(21))
}

test.setup_expression_check_identical <- function()
{
  set.seed(21)
  x <- rnorm(1e5)
  microbenchmark(rnorm(1e5), x, check = 'identical', setup = set.seed(21))
}

test.setup_expression_check_identical_failure <- function()
{
  set.seed(21)
  x <- rnorm(1e5)
  attr(x = x, 'abc') <- 123 # add attribute
  out <- try(microbenchmark(rnorm(1e5), x, check = 'equal', setup = set.seed(21)), silent = T)
  checkTrue(inherits(out, "try-error"))
}

test.print_returns_input <- function()
{
  x <- microbenchmark( 5 + 6, 6 + 7, times = 2)
  identical(x, print(x))
}

#' Return the current value of the platform timer.
#'
#' The current value of the most accurate timer of the platform is
#' returned. This can be used as a time stamp for logging or similar
#' purposes. Please note that there is no common reference, that is,
#' the timer value cannot be converted to a date and time value.
#' 
#' @author Olaf Mersmann
get_nanotime <- function() {
  .Call(do_get_nanotime, PACKAGE="microbenchmark")
}

#' Summarize \code{microbenchmark} timings.
#'
#' @param object An object of class \code{microbenchmark}.
#'
#' @param unit What unit to print the timings in. If none is given,
#' either the \code{unit} attribute of \code{object} or the option
#' \code{microbenchmark.unit} is used and if neither is set
#' \dQuote{t} is used.
#'
#' @param ... Ignored
#'
#' @param include_cld Calculate \code{cld} using \code{multcomp::glht()}
#' and add it to the output. Set to \code{FALSE} if the calculation takes
#' too long.
#'
#' @note The available units are nanoseconds (\code{"ns"}),
#' microseconds (\code{"us"}), milliseconds (\code{"ms"}), seconds
#' (\code{"s"}) and evaluations per seconds (\code{"eps"}) and
#' relative runtime compared to the best median time
#' (\code{"relative"}).
#'
#' @return A \code{data.frame} containing the aggregated results.
#'
#' @seealso \code{\link{print.microbenchmark}}
#' @method summary microbenchmark
summary.microbenchmark <- function(object, unit, ..., include_cld = TRUE) {
  ## Choose unit if not given based on unit attribute of object or
  ## global option. Default to 't' if none is set.
  if (missing(unit)) {
    unit <- NULL
  }
  unit <- determine_unit(object, unit)

  if (unit != "relative")
    object$time <- convert_to_unit(object, unit)

  res <- aggregate(time ~ expr, object,
                   function(z) {
                       tmp <- c(fivenum(z), mean(z), length(z))
                       tmp[c(1, 2, 6, 3, 4, 5, 7)]
                   })
  res <- cbind(res$expr, as.data.frame(res$time))
  colnames(res) <- c("expr", "min", "lq", "mean", "median", "uq", "max", "neval")

  if (unit == "relative") {
    min <- res[which.min(res$median), , drop = FALSE]
    min$neval <- 1 # Ugly hack: Do not rescale neval
    res[-1] <- res[-1] / as.list(min[-1])
    attr(res, "unit") <- "relative"
  } else {
    attr(res, "unit") <- attr(object$time, "unit")
  }

  if (isTRUE(include_cld) && requireNamespace("multcomp", quietly = TRUE)
      && nrow(res) > 1 && all(res["neval"] > 1)) {
    ## Try to calculate a statistically meaningful comparison. If it fails for
    ## any reason (f.e. the data might be constant), ignore the error.
    cld_time <- system.time({
      tryCatch({
        ops <- options(warn=-1)
        mdl <- lm(time ~ expr, object)
        comp <- multcomp::glht(mdl, multcomp::mcp(expr = "Tukey"))
        res$cld <- multcomp::cld(comp)$mcletters$monospacedLetters
      }, error=function(e) FALSE, finally=options(ops))
    })
    if (cld_time["elapsed"] > 5.0) {
      message("cld calculation took more than 5 seconds\n  set",
              " include_cld = FALSE to skip the cld calculation")
    }
  }
  res
}

register_s3_method <-
function(pkg, generic, class, fun = NULL)
{
  stopifnot(is.character(pkg), length(pkg) == 1L)
  stopifnot(is.character(generic), length(generic) == 1L)
  stopifnot(is.character(class), length(class) == 1L)

  if (is.null(fun)) {
    fun <- get(paste0(generic, ".", class), envir = parent.frame())
  } else {
    stopifnot(is.function(fun))
  }

  if (isNamespaceLoaded(pkg)) {
    registerS3method(generic, class, fun, envir = asNamespace(pkg))
  }

  # Always register hook in case package is later unloaded & reloaded
  setHook(
    packageEvent(pkg, "onLoad"),
    function(...) {
      registerS3method(generic, class, fun, envir = asNamespace(pkg))
    }
  )
}

.onLoad <- function(libname, pkgname) {
  if (getRversion() < "3.6.0") {
    register_s3_method("ggplot2", "autoplot", "microbenchmark")
  }
  invisible()
}

.onUnload <- function(libpath) {
  library.dynam.unload("microbenchmark", libpath)
}

#' Print \code{microbenchmark} timings.
#' 
#' @param x An object of class \code{microbenchmark}.
#' @param unit What unit to print the timings in. Default value taken
#'   from to option \code{microbenchmark.unit} (see example).
#' @param order If present, order results according to this column of the output.
#' @param signif If present, limit the number of significant digits shown.
#' @param ... Passed to \code{print.data.frame}
#'
#' @note The available units are nanoseconds (\code{"ns"}), microseconds
#' (\code{"us"}), milliseconds (\code{"ms"}), seconds (\code{"s"})
#' and evaluations per seconds (\code{"eps"}) and relative runtime
#' compared to the best median time (\code{"relative"}).
#'
#' @note If the \code{multcomp} package is available a statistical
#' ranking is calculated and displayed in compact letter display from
#' in the \code{cld} column.
#' 
#' @seealso \code{\link{boxplot.microbenchmark}} and
#' \code{\link{autoplot.microbenchmark}} for a plot methods.
#'
#' @examples
#' a1 <- a2 <- a3 <- a4 <- numeric(0)
#' 
#' res <- microbenchmark(a1 <- c(a1, 1),
#'                       a2 <- append(a2, 1),
#'                       a3[length(a3) + 1] <- 1,
#'                       a4[[length(a4) + 1]] <- 1,
#'                       times=100L)
#' print(res)
#' ## Change default unit to relative runtime
#' options(microbenchmark.unit="relative")
#' print(res)
#' ## Change default unit to evaluations per second
#' options(microbenchmark.unit="eps")
#' print(res)
#'
#' @method print microbenchmark
#' @author Olaf Mersmann
print.microbenchmark <- function(x, unit, order, signif, ...) {
  if (missing(unit)) {
    unit <- NULL
  }
  unit <- determine_unit(x, unit)
  s <- summary(x, unit=unit)
  timing_cols <- c("min", "lq", "median", "uq", "max", "mean")
  if (!missing(signif)) {
    s[timing_cols] <- lapply(s[timing_cols], base::signif, signif)
  }
  cat("Unit: ", attr(s, "unit"), "\n", sep="")
  if (!missing(order)) {
    if (order %in% colnames(s)) {
      s <- s[order(s[[order]]), ]
    } else {
      warning("Cannot order results by", order, ".")
    }
  }
  print(s, ..., row.names=FALSE)
  invisible(x)
}

#' @method rbind microbenchmark
rbind.microbenchmark <-
function(..., deparse.level = 1)
{
    args <- list(...)
    is_mb <- sapply(args, inherits, "microbenchmark")
    if(!all(is_mb)) {
        stop("can only combine microbenchmark objects")
    }

    output <- args
    arg_names <- as.character(substitute(alist(...))[-1])
    all_levels <- NULL
    for(i in seq_along(output)) {
        # expressions and levels for object 'i'
        i_expr <- output[[i]]$expr
        i_levels <- levels(i_expr)
        # all levels so far...
        all_levels <- c(all_levels, i_levels)
        # add object name to suffix for duplicated expressions
        has_dups <- utils::tail(duplicated(all_levels), -nlevels(i_expr))
        if (any(has_dups)) {
            levels(output[[i]]$expr) <- paste(i_levels, arg_names[i], sep = ".")
        }
    }
    output <- do.call("rbind.data.frame", output)
    class(output) <- class(args[[1]])
    return(output)
}


    #arg_names <- names(args)  # if called via do.call()
    #arg_symbols <- as.character(substitute(alist(...))[-1])
    #if (is.null(arg_names)) {
    #    arg_names <- arg_symbols
    #}
    #no_name <- arg_names == ""
    #arg_names[no_name] <- seq_along(arg_names)[no_name]

    #all_expr <- lapply(args, function(x) x$expr)
    #all_levels <- sapply(all_expr, levels)
    #is_dupe <- duplicated(all_levels)
    #positions <- rep(seq_along(args), times = sapply(all_expr, nlevels))
    #unique_levels <- make.unique(all_levels)

#' Sub-millisecond accurate timing of expression evaluation.
#'
#' \code{microbenchmark} serves as a more accurate replacement of the
#' often seen \code{system.time(replicate(1000, expr))}
#' expression. It tries hard to accurately measure only the time it
#' takes to evaluate \code{expr}. To achieved this, the
#' sub-millisecond (supposedly nanosecond) accurate timing functions
#' most modern operating systems provide are used. Additionally all
#' evaluations of the expressions are done in C code to minimize any
#' overhead.
#'
#' This function is only meant for micro-benchmarking small pieces of
#' source code and to compare their relative performance
#' characteristics. You should generally avoid benchmarking larger
#' chunks of your code using this function. Instead, try using the R
#' profiler to detect hot spots and consider rewriting them in C/C++
#' or FORTRAN.
#'
#' The \code{control} list can contain the following entries:
#' \describe{
#' \item{order}{the order in which the expressions are evaluated.
#'   \dQuote{random} (the default) randomizes the execution order,
#'   \dQuote{inorder} executes each expression in order and
#'   \dQuote{block} executes all repetitions of each expression
#'     as one block.}
#' \item{warmup}{the number of iterations to run the timing code before
#'   evaluating the expressions in \dots. These warm-up iterations are used
#'   to estimate the timing overhead as well as spinning up the processor
#'   from any sleep or idle states it might be in. The default value is 2.}
#' }
#'
#' @note Depending on the underlying operating system, different
#' methods are used for timing. On Windows the
#' \code{QueryPerformanceCounter} interface is used to measure the
#' time passed. For Linux the \code{clock_gettime} API is used and on
#' Solaris the \code{gethrtime} function. Finally on MacOS X the,
#' undocumented, \code{mach_absolute_time} function is used to avoid
#' a dependency on the CoreServices Framework.
#'
#' Before evaluating each expression \code{times} times, the overhead
#' of calling the timing functions and the C function call overhead
#' are estimated. This estimated overhead is subtracted from each
#' measured evaluation time. Should the resulting timing be negative,
#' a warning is thrown and the respective value is replaced by
#' \code{0}. If the timing is zero, a warning is raised.
#' Should all evaluations result in one of the two error conditions described above, an error is raised.
#'
#' One platform on which the clock resolution is known to be too low to measure short runtimes with the required precision is 
#' Oracle\if{html}{\out{&reg;}}\if{latex}{\out{\textregistered\ }}\if{text}{(R)}
#' Solaris 
#' on some 
#' SPARC\if{html}{\out{&reg;}}\if{latex}{\out{\textregistered\ }}\if{text}{(R)}
#' hardware.
#' Reports of other platforms with similar problems are welcome.
#' Please contact the package maintainer.
#'
#' @param ... Expressions to benchmark.
#' @param list  List of unevaluated expressions to benchmark.
#' @param times Number of times to evaluate each expression.
#' @param check A function to check if the expressions are equal. By default \code{NULL} which omits the check.
#' In addition to a function, a string can be supplied.
#' The string \sQuote{equal} will compare all values using \code{\link{all.equal}}, \sQuote{equivalent} will compare all values using \code{\link{all.equal}} and check.attributes = FALSE, and \sQuote{identical} will compare all values using \code{\link{identical}}.
#' @param control List of control arguments. See Details.
#' @param unit Default unit used in \code{summary} and \code{print}.
#' @param setup An unevaluated expression to be run (untimed) before each benchmark expression.
#'
#' @return Object of class \sQuote{microbenchmark}, a data frame with
#' columns \code{expr} and \code{time}. \code{expr} contains the
#' deparsed expression as passed to \code{microbenchmark} or the name
#' of the argument if the expression was passed as a named
#' argument. \code{time} is the measured execution time of the
#' expression in nanoseconds. The order of the observations in the
#' data frame is the order in which they were executed.
#'
#' @seealso \code{\link{print.microbenchmark}} to display and
#' \code{\link{boxplot.microbenchmark}} or
#' \code{\link{autoplot.microbenchmark}} to plot the results.
#'
#' @examples
#' ## Measure the time it takes to dispatch a simple function call
#' ## compared to simply evaluating the constant \code{NULL}
#' f <- function() NULL
#' res <- microbenchmark(NULL, f(), times=1000L)
#'
#' ## Print results:
#' print(res)
#'
#' ## Plot results:
#' boxplot(res)
#'
#' ## Pretty plot:
#' if (requireNamespace("ggplot2")) {
#'   ggplot2::autoplot(res)
#' }
#'
#' ## Example check usage
#' my_check <- function(values) {
#'   all(sapply(values[-1], function(x) identical(values[[1]], x)))
#' }
#'
#' f <- function(a, b)
#'   2 + 2
#'
#' a <- 2
#' ## Check passes
#' microbenchmark(2 + 2, 2 + a, f(2, a), f(2, 2), check=my_check)
#' \dontrun{
#' a <- 3
#' ## Check fails
#' microbenchmark(2 + 2, 2 + a, f(2, a), f(2, 2), check=my_check)
#' }
#' ## Example setup usage
#' set.seed(21)
#' x <- rnorm(10)
#' microbenchmark(x, rnorm(10), check=my_check, setup=set.seed(21))
#' ## Will fail without setup
#' \dontrun{
#' microbenchmark(x, rnorm(10), check=my_check)
#' }
#' ## using check
#' a <- 2
#' microbenchmark(2 + 2, 2 + a, sum(2, a), sum(2, 2), check='identical')
#' microbenchmark(2 + 2, 2 + a, sum(2, a), sum(2, 2), check='equal')
#' attr(a, 'abc') <- 123
#' microbenchmark(2 + 2, 2 + a, sum(2, a), sum(2, 2), check='equivalent')
#' ## check='equal' will fail due to difference in attribute
#' \dontrun{
#' microbenchmark(2 + 2, 2 + a, sum(2, a), sum(2, 2), check='equal')
#' }
#' @author Olaf Mersmann
microbenchmark <- function(..., list=NULL,
                           times=100L,
                           unit=NULL,
                           check=NULL,
                           control=list(),
                           setup=NULL) {
  stopifnot(times == as.integer(times))
  if (!missing(unit) && !is.null(unit))
    stopifnot(is.character(unit), length(unit) == 1L)

  unit <- determine_unit(unit = unit)

  control[["warmup"]] <- coalesce(control[["warmup"]], 2^18L)
  control[["order"]] <- coalesce(control[["order"]], "random")

  stopifnot(as.integer(control$warmup) == control$warmup)

  exprs <- c(as.list(match.call(expand.dots = FALSE)$`...`), list)
  nm <- names(exprs)
  exprnm <- sapply(exprs, function(e) paste(deparse(e), collapse=" "))
  if (is.null(nm))
    nm <- exprnm
  else
    nm[nm == ""] <- exprnm[nm == ""]
  names(exprs) <- nm

  env <- parent.frame()
  setup <- substitute(setup)

  if (!is.null(check)) {
    setupexpr <- as.expression(setup)
    checkexprs <- lapply(exprs, function(e) c(setupexpr, e))

    ## Evaluate values in parent environment
    values <- lapply(checkexprs, eval, env)
    if (is.character(check) && isTRUE(check == 'equal')) {
      check <- function(values) { all(sapply(values[-1], function(x) isTRUE(all.equal(values[[1]], x)))) }
    } else if (is.character(check) && isTRUE(check == 'equivalent')) {
      check <- function(values) { all(sapply(values[-1], function(x) isTRUE(all.equal(values[[1]], x, check.attributes = F)))) }
    } else if (is.character(check) && isTRUE(check == 'identical')) {
      check <- function(values) { all(sapply(values[-1], function(x) identical(values[[1]], x))) }
    }
    ok <- check(values)

    if (!isTRUE(ok)) {
      stop("Input expressions are not equivalent.", call. = FALSE)
    }
  }

  ## GC first
  gc(FALSE)

  o <- if (control$order == "random")
    sample(rep(seq_along(exprs), times=times))
  else if (control$order == "inorder")
    rep(seq_along(exprs), times=times)
  else if (control$order == "block")
    rep(seq_along(exprs), each=times)
  else
    stop("Unknown ordering. Must be one of 'random', 'inorder' or 'block'.")
  exprs <- exprs[o]

  if (anyDuplicated(nm) > 0) {
    duplicates <- nm[duplicated(nm)]
    stop("Expression names must be unique. Duplicate expression names: ",
         paste(duplicates, collapse = ", "))
  }
  expr <- factor(nm[o], levels = nm)
  res <- .Call(do_microtiming, exprs, env,
               as.integer(control$warmup), setup,
               PACKAGE="microbenchmark")

  ## Sanity check. Fail as early as possible if the results are
  ## rubbish.
  if (all(is.na(res)))
    .all_na_stop()

  res <- data.frame(expr = expr, time=res)
  class(res) <- c("microbenchmark", class(res))
  if (!is.null(unit))
    attr(res, "unit") <- unit
  res
}

## Internal utility functions

# Internal helper functions that returns a generic error if timings fail.
.all_na_stop <- function() {
  msg <- "All measured timings are NA. This is bad!

There are several causes for this. The most likely are

 * You are running under a hypervisor. This can do all sorts of things
   to the timing functions of your operating system.

 * You have frequency scaling turned on. Most modern CPUs can reduce
   their core frequency if they are not busy. microbenchmark tries
   hard to spin up the CPU before the actual timing, but there is no
   guarantee this works, so you are advised to disable this
   feature. Under Linux this can be done using the 'cpufreq'
   utilities.

 * You have a machine with many CPU cores and the timers provided by
   your operating system are not synchronized across cores. Your best
   bet is to peg your R process to a single core. On Linux systems,
   this can be achieved using the 'taskset' utility.

 * Your machine is super fast. If the difference between the estimated
   overhead and the actual execution time is zero (or possibly even
   negative), you will get this error. Sorry,
   you're out of luck, benchmark more complex code.

If this problem persists for you, please contact me and I will try to
resolve the issue with you."

  stop(msg, call.=FALSE)
}

#' Convert timings to different units.
#'
#' The following units of time are supported \describe{
#' \item{\dQuote{ns}}{Nanoseconds.}
#' \item{\dQuote{us}}{Microseconds.}
#' \item{\dQuote{ms}}{Milliseconds.}
#' \item{\dQuote{s}}{Seconds.}
#' \item{\dQuote{t}}{Appropriately prefixed time unit.}
#' \item{\dQuote{hz}}{Hertz / evaluations per second.}
#' \item{\dQuote{eps}}{Evaluations per second / Hertz.}
#' \item{\dQuote{khz}}{Kilohertz / 1000s of evaluations per second.}
#' \item{\dQuote{mhz}}{Megahertz / 1000000s of evaluations per second.}
#' \item{\dQuote{f}}{Appropriately prefixed frequency unit.}
#' }
#' 
#' @param object A \code{microbenchmark} object.
#' @param unit A unit of time. See details.
#'
#' @return A matrix containing the converted time values with an
#'   attribute \code{unit} which is a printable name of the unit of
#'   time.
#'
#' @author Olaf Mersmann
#'
#' @keywords internal
convert_to_unit <- function(object, unit)
{
  unit <- determine_unit(object, unit)
  x <- object$time

  switch (unit,
          t=unit <- sprintf ("%ss", find_prefix(x * 1e-9,
            minexp = -9, maxexp = 0, mu = FALSE)),
          f=unit <- sprintf ("%shz", find_prefix(1e9 / x,
            minexp =  0, maxexp = 6, mu = FALSE))
          )
  unit <- tolower(unit)
  switch (unit,
          ns  ={attr(x, "unit") <- "nanoseconds"           ; unclass(x      )},
          us  ={attr(x, "unit") <- "microseconds"          ; unclass(x / 1e3)},
          ms  ={attr(x, "unit") <- "milliseconds"          ; unclass(x / 1e6)}, 
          s   ={attr(x, "unit") <- "seconds"               ; unclass(x / 1e9)},
          eps ={attr(x, "unit") <- "evaluations per second"; unclass(1e9 / x)},
          hz  ={attr(x, "unit") <- "hertz"                 ; unclass(1e9 / x)},
          khz ={attr(x, "unit") <- "kilohertz"             ; unclass(1e6 / x)},
          mhz ={attr(x, "unit") <- "megahertz"             ; unclass(1e3 / x)},
          stop("Unknown unit '", unit, "'.")
          )
}

#' Find SI prefix for unit
#' 
#' @param x a numeric
#' @param f function that produces the number from \code{x} that is used to
#'   determine the prefix, e.g. \code{\link[base]{min}} or
#'   \code{\link[stats]{median}}.
#' @param minexp minimum (decimal) exponent to consider, 
#'   e.g. -3 to suppress prefixes smaller than milli (m).
#' @param maxexp maximum (decimal) exponent to consider, 
#'   e.g. 3 to suppress prefixes larger than kilo (k).
#' @param mu if \code{TRUE}, should a proper mu be used for micro, otherwise use
#'  u as ASCII-compatible replacement
#'
#' @return character with the SI prefix
#' @author Claudia Beleites 
#'
#' @keywords internal
find_prefix <- function (x, f=min, minexp=-Inf, maxexp=Inf, mu=TRUE) {
  prefixes <- c ("y", "z", "a", "f", "p", "n", "u", "m", "",
                 "k", "M", "G", "T", "P", "E", "Z", "Y")
  if (mu) prefixes [7] <- "\u03bc"

  if (is.numeric (minexp)) minexp <- floor (minexp / 3) 
  if (is.numeric (minexp)) maxexp <- floor (maxexp / 3) 

  e3 <- floor (log10 (f (x)) / 3)
  e3 <- max (e3, minexp, -8) # prefixes go from 10^-24 = 10^(3 * -8)
  e3 <- min (e3, maxexp,  8) # to 10^24 = 10^(3 * 8)

  prefixes [e3 + 9] # e3 of -8 => index 1
 }

#' Return first non null argument.
#'
#' This function is useful when processing complex arguments with multiple
#' possible defaults based on other arguments that may or may not have been
#' provided.
#'
#' @param ... List of values.
#' @return First non null element in \code{...}.
#'
#' @author Olaf Mersmann
#'
#' @keywords internal
coalesce <- function(...) {
  isnotnull <- function(x) !is.null(x)
  Find(isnotnull, list(...))
}

#' Normalize timing units to one of the supported values
#'
#' We support the following units of time
#' \describe{
#'   \item{\dQuote{ns}, \dQuote{nanoseconds}}{}
#'   \item{\dQuote{us}, \dQuote{microseconds}}{}
#'   \item{\dQuote{ms}, \dQuote{milliseconds}}{}
#'   \item{\dQuote{s}, \dQuote{secs}, \dQuote{seconds}}{}
#'   \item{\dQuote{t}, \dQuote{time}}{Appropriately prefixed time unit.}
#'   \item{\dQuote{eps}}{Evaluations per second / Hertz.}
#'   \item{\dQuote{hz}}{Hertz / evaluations per second.}
#'   \item{\dQuote{khz}}{Kilohertz / 1000s of evaluations per second.}
#'   \item{\dQuote{mhz}}{Megahertz / 1000000s of evaluations per second.}
#'   \item{\dQuote{f}, \dQuote{frequency}}{Appropriately prefixed frequency unit.}
#' }
#'
#' @param object A 'microbenchmark' object.
#' @param unit A unit of time. See details.
#'
#' @return A matrix containing the converted time values with an
#'   attribute \code{unit} which is a printable name of the unit of
#'   time.
#'
#' @author Joshua M. Ulrich
#'
#' @keywords internal
determine_unit <-
function(object = NULL,
         unit = NULL)
{
  # all supported unit values
  values <- c("nanoseconds", "ns",
              "microseconds", "us",
              "milliseconds", "ms",
              "seconds",
              "time", "auto",
              "frequency",
              "hz", "khz", "mhz",
              "eps", "relative")

  # Order of precedence:
  # 1) 'unit' argument
  # 2) 'unit' attribute on 'object' argument
  # 3) 'microbenchmark.unit' option
  object_unit <- attr(object, "unit")
  if (is.null(unit)) {
    if (is.null(object_unit)) {
        unit <- getOption("microbenchmark.unit", "auto")
    } else {
        unit <- object_unit
    }
  } else {
    # include support for 'secs' because it doesn't match 'seconds'
    unit <- if (unit == "secs") "seconds" else unit
  }

  unit <- tolower(unit)
  unit <- match.arg(unit, values)

  unit <-
    switch(unit,
           nanoseconds = ,
           ns = "ns",
           microseconds = ,
           us = "us",
           milliseconds = ,
           ms = "ms",
           seconds = "s",
           auto = ,
           time = "t",
           frequency = "f",
           hz = "hz",
           khz = "khz",
           mhz = "mhz",
           eps = "eps",
           relative = "relative")
  unit
}

#' Boxplot of \code{microbenchmark} timings.
#'
#' @param x A \code{microbenchmark} object.
#' @param unit Unit in which the results be plotted.
#' @param log Should times be plotted on log scale?
#' @param xlab X axis label.
#' @param ylab Y axis label.
#' @param horizontal Switch X and Y axes.
#' @param main Plot title.
#' @param ... Passed on to boxplot.formula.
#' 
#' @method boxplot microbenchmark
#' 
#' @author Olaf Mersmann
boxplot.microbenchmark <- function(x, unit="t", log=TRUE, xlab, ylab, 
                                   horizontal=FALSE, main="microbenchmark timings", ...) {
  x$time <- convert_to_unit(x, unit)
  timeunits <- c("ns", "us", "ms", "s", "t")
  frequnits <- c("hz", "khz", "mhz", "eps", "f")

  if (missing(xlab))
    xlab <- "Expression"
  if (missing(ylab)) {
    ylab <- if (log) {
      if (unit %in% timeunits)
        paste("log(time) [", unit, "]", sep="")
      else if (unit %in% frequnits)
        paste("log(frequency) [", unit, "]", sep="")
      else
        paste("log(", unit, ")", sep="")
    } else {
      if (unit %in% timeunits)
        paste("time [", unit, "]", sep="")
      else if (unit %in% frequnits)
        paste("frequency [", unit, "]", sep="")
      else if (unit == "eps")
        "evaluations per second [Hz]"
      else
        unit
    }
  }

  if (log) {
    # min time cannot be 0 for log y-axis
    ylim <- pmax(1, range(x$time))
  } else {
    ylim <- NULL
  }
  if (horizontal) { 
    ll <- if (log) "x" else ""
    boxplot(time ~ expr, data=x, xlab=ylab, ylab=xlab, log=ll, ylim=ylim,
            horizontal=TRUE, main=main, ...)
  } else {
    ll <- if (log) "y" else ""
    boxplot(time ~ expr, data=x, xlab=xlab, ylab=ylab, log=ll, ylim=ylim, main=main, ...)
  }
}

#' Estimate precision of timing routines.
#'
#' This function is currently experimental. Its main use is to judge
#' the quality of the underlying timer implementation of the
#' operating system. The function measures the overhead of timing a C
#' function call \code{rounds} times and returns all non-zero timings
#' observed. This can be used to judge the granularity and resolution
#' of the timing subsystem.
#' 
#' @param rounds Number of measurements used to estimate the precision.
#' @param warmup Number of iterations used to warm up the CPU.
#' @return A vector of observed non-zero timings.
#'
#' @author Olaf Mersmann
microtiming_precision <- function(rounds=100L, warmup=2^18) {
  .Call(do_microtiming_precision, parent.frame(),
        as.integer(rounds),
        as.integer(warmup),
        PACKAGE="microbenchmark")
}

#' Autoplot method for microbenchmark objects: Prettier graphs for
#' microbenchmark using ggplot2
#'
#' Uses ggplot2 to produce a more legible graph of microbenchmark timings.
#'
#' @param object A microbenchmark object.
#' @param \dots Ignored.
#' @param order Names of output column(s) to order the results.
#' @param log If \code{TRUE} the time axis will be on log scale.
#' @param unit The unit to use for graph labels.
#' @param y_max The upper limit of the y axis, in the unit automatically
#'   chosen for the time axis (defaults to the maximum value).
#' @return A ggplot2 object.
#'
#' @examples
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'     tm <- microbenchmark(rchisq(100, 0),
#'                          rchisq(100, 1),
#'                          rchisq(100, 2),
#'                          rchisq(100, 3),
#'                          rchisq(100, 5), times=1000L)
#'     ggplot2::autoplot(tm)
#'
#'     # add a custom title
#'     ggplot2::autoplot(tm) + ggplot2::ggtitle("my timings")
#' }
#' @author Ari Friedman, Olaf Mersmann
autoplot.microbenchmark <- function(object, ...,
                                    order=NULL,
                                    log=TRUE,
                                    unit=NULL,
                                    y_max=NULL) {
  if (!requireNamespace("ggplot2", quietly = TRUE))
    stop("Missing package 'ggplot2'.")
  y_min <- 0

  unit <- determine_unit(object, unit)
  object$ntime <- convert_to_unit(object, unit)
  if (is.null(y_max)) {
    y_max <- max(object$ntime)
  }
  if (!is.null(order)) {
    s <- summary(object)
    object_colnames <- colnames(s)
    order <- match.arg(order, object_colnames, several.ok=TRUE)
    new_order <- do.call("order", c(s[, order, drop=FALSE], decreasing=TRUE))
    object$expr <- factor(object$expr, levels = levels(object$expr)[new_order])
  }
  plt <- ggplot2::ggplot(object, ggplot2::aes_string(x="expr", y="ntime"))
  plt <- plt + ggplot2::stat_ydensity()
  plt <- plt + ggplot2::scale_x_discrete(name="")

  y_label <- sprintf("Time (%s) for neval = %d",
                     attr(object$ntime, "unit"),
                     nrow(object) / length(levels(object$expr)))

  if (log) {
    y_min <- if (min(object$time) == 0) 1 else min(object$ntime)
    plt <- plt + ggplot2::scale_y_log10(name=y_label)
  } else {
    plt <- plt + ggplot2::scale_y_continuous(name=y_label)
  }
  plt <- plt + ggplot2::coord_flip(ylim=c(y_min , y_max))
  plt <- plt + ggplot2::ggtitle("microbenchmark timings")
  plt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autoplot.R
\name{autoplot.microbenchmark}
\alias{autoplot.microbenchmark}
\title{Autoplot method for microbenchmark objects: Prettier graphs for
microbenchmark using ggplot2}
\usage{
autoplot.microbenchmark(
  object,
  ...,
  order = NULL,
  log = TRUE,
  unit = NULL,
  y_max = NULL
)
}
\arguments{
\item{object}{A microbenchmark object.}

\item{\dots}{Ignored.}

\item{order}{Names of output column(s) to order the results.}

\item{log}{If \code{TRUE} the time axis will be on log scale.}

\item{unit}{The unit to use for graph labels.}

\item{y_max}{The upper limit of the y axis, in the unit automatically
chosen for the time axis (defaults to the maximum value).}
}
\value{
A ggplot2 object.
}
\description{
Uses ggplot2 to produce a more legible graph of microbenchmark timings.
}
\examples{
if (requireNamespace("ggplot2", quietly = TRUE)) {
    tm <- microbenchmark(rchisq(100, 0),
                         rchisq(100, 1),
                         rchisq(100, 2),
                         rchisq(100, 3),
                         rchisq(100, 5), times=1000L)
    ggplot2::autoplot(tm)

    # add a custom title
    ggplot2::autoplot(tm) + ggplot2::ggtitle("my timings")
}
}
\author{
Ari Friedman, Olaf Mersmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal.R
\name{convert_to_unit}
\alias{convert_to_unit}
\title{Convert timings to different units.}
\usage{
convert_to_unit(object, unit)
}
\arguments{
\item{object}{A \code{microbenchmark} object.}

\item{unit}{A unit of time. See details.}
}
\value{
A matrix containing the converted time values with an
  attribute \code{unit} which is a printable name of the unit of
  time.
}
\description{
The following units of time are supported \describe{
\item{\dQuote{ns}}{Nanoseconds.}
\item{\dQuote{us}}{Microseconds.}
\item{\dQuote{ms}}{Milliseconds.}
\item{\dQuote{s}}{Seconds.}
\item{\dQuote{t}}{Appropriately prefixed time unit.}
\item{\dQuote{hz}}{Hertz / evaluations per second.}
\item{\dQuote{eps}}{Evaluations per second / Hertz.}
\item{\dQuote{khz}}{Kilohertz / 1000s of evaluations per second.}
\item{\dQuote{mhz}}{Megahertz / 1000000s of evaluations per second.}
\item{\dQuote{f}}{Appropriately prefixed frequency unit.}
}
}
\author{
Olaf Mersmann
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/microtiming_precision.R
\name{microtiming_precision}
\alias{microtiming_precision}
\title{Estimate precision of timing routines.}
\usage{
microtiming_precision(rounds = 100L, warmup = 2^18)
}
\arguments{
\item{rounds}{Number of measurements used to estimate the precision.}

\item{warmup}{Number of iterations used to warm up the CPU.}
}
\value{
A vector of observed non-zero timings.
}
\description{
This function is currently experimental. Its main use is to judge
the quality of the underlying timer implementation of the
operating system. The function measures the overhead of timing a C
function call \code{rounds} times and returns all non-zero timings
observed. This can be used to judge the granularity and resolution
of the timing subsystem.
}
\author{
Olaf Mersmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.microbenchmark}
\alias{print.microbenchmark}
\title{Print \code{microbenchmark} timings.}
\usage{
\method{print}{microbenchmark}(x, unit, order, signif, ...)
}
\arguments{
\item{x}{An object of class \code{microbenchmark}.}

\item{unit}{What unit to print the timings in. Default value taken
from to option \code{microbenchmark.unit} (see example).}

\item{order}{If present, order results according to this column of the output.}

\item{signif}{If present, limit the number of significant digits shown.}

\item{...}{Passed to \code{print.data.frame}}
}
\description{
Print \code{microbenchmark} timings.
}
\note{
The available units are nanoseconds (\code{"ns"}), microseconds
(\code{"us"}), milliseconds (\code{"ms"}), seconds (\code{"s"})
and evaluations per seconds (\code{"eps"}) and relative runtime
compared to the best median time (\code{"relative"}).

If the \code{multcomp} package is available a statistical
ranking is calculated and displayed in compact letter display from
in the \code{cld} column.
}
\examples{
a1 <- a2 <- a3 <- a4 <- numeric(0)

res <- microbenchmark(a1 <- c(a1, 1),
                      a2 <- append(a2, 1),
                      a3[length(a3) + 1] <- 1,
                      a4[[length(a4) + 1]] <- 1,
                      times=100L)
print(res)
## Change default unit to relative runtime
options(microbenchmark.unit="relative")
print(res)
## Change default unit to evaluations per second
options(microbenchmark.unit="eps")
print(res)

}
\seealso{
\code{\link{boxplot.microbenchmark}} and
\code{\link{autoplot.microbenchmark}} for a plot methods.
}
\author{
Olaf Mersmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal.R
\name{coalesce}
\alias{coalesce}
\title{Return first non null argument.}
\usage{
coalesce(...)
}
\arguments{
\item{...}{List of values.}
}
\value{
First non null element in \code{...}.
}
\description{
This function is useful when processing complex arguments with multiple
possible defaults based on other arguments that may or may not have been
provided.
}
\author{
Olaf Mersmann
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal.R
\name{determine_unit}
\alias{determine_unit}
\title{Normalize timing units to one of the supported values}
\usage{
determine_unit(object = NULL, unit = NULL)
}
\arguments{
\item{object}{A 'microbenchmark' object.}

\item{unit}{A unit of time. See details.}
}
\value{
A matrix containing the converted time values with an
  attribute \code{unit} which is a printable name of the unit of
  time.
}
\description{
We support the following units of time
\describe{
  \item{\dQuote{ns}, \dQuote{nanoseconds}}{}
  \item{\dQuote{us}, \dQuote{microseconds}}{}
  \item{\dQuote{ms}, \dQuote{milliseconds}}{}
  \item{\dQuote{s}, \dQuote{secs}, \dQuote{seconds}}{}
  \item{\dQuote{t}, \dQuote{time}}{Appropriately prefixed time unit.}
  \item{\dQuote{eps}}{Evaluations per second / Hertz.}
  \item{\dQuote{hz}}{Hertz / evaluations per second.}
  \item{\dQuote{khz}}{Kilohertz / 1000s of evaluations per second.}
  \item{\dQuote{mhz}}{Megahertz / 1000000s of evaluations per second.}
  \item{\dQuote{f}, \dQuote{frequency}}{Appropriately prefixed frequency unit.}
}
}
\author{
Joshua M. Ulrich
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanotime.R
\name{get_nanotime}
\alias{get_nanotime}
\title{Return the current value of the platform timer.}
\usage{
get_nanotime()
}
\description{
The current value of the most accurate timer of the platform is
returned. This can be used as a time stamp for logging or similar
purposes. Please note that there is no common reference, that is,
the timer value cannot be converted to a date and time value.
}
\author{
Olaf Mersmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.microbenchmark}
\alias{summary.microbenchmark}
\title{Summarize \code{microbenchmark} timings.}
\usage{
\method{summary}{microbenchmark}(object, unit, ..., include_cld = TRUE)
}
\arguments{
\item{object}{An object of class \code{microbenchmark}.}

\item{unit}{What unit to print the timings in. If none is given,
either the \code{unit} attribute of \code{object} or the option
\code{microbenchmark.unit} is used and if neither is set
\dQuote{t} is used.}

\item{...}{Ignored}

\item{include_cld}{Calculate \code{cld} using \code{multcomp::glht()}
and add it to the output. Set to \code{FALSE} if the calculation takes
too long.}
}
\value{
A \code{data.frame} containing the aggregated results.
}
\description{
Summarize \code{microbenchmark} timings.
}
\note{
The available units are nanoseconds (\code{"ns"}),
microseconds (\code{"us"}), milliseconds (\code{"ms"}), seconds
(\code{"s"}) and evaluations per seconds (\code{"eps"}) and
relative runtime compared to the best median time
(\code{"relative"}).
}
\seealso{
\code{\link{print.microbenchmark}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal.R
\name{find_prefix}
\alias{find_prefix}
\title{Find SI prefix for unit}
\usage{
find_prefix(x, f = min, minexp = -Inf, maxexp = Inf, mu = TRUE)
}
\arguments{
\item{x}{a numeric}

\item{f}{function that produces the number from \code{x} that is used to
determine the prefix, e.g. \code{\link[base]{min}} or
\code{\link[stats]{median}}.}

\item{minexp}{minimum (decimal) exponent to consider, 
e.g. -3 to suppress prefixes smaller than milli (m).}

\item{maxexp}{maximum (decimal) exponent to consider, 
e.g. 3 to suppress prefixes larger than kilo (k).}

\item{mu}{if \code{TRUE}, should a proper mu be used for micro, otherwise use
u as ASCII-compatible replacement}
}
\value{
character with the SI prefix
}
\description{
Find SI prefix for unit
}
\author{
Claudia Beleites
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/microbenchmark.R
\name{microbenchmark}
\alias{microbenchmark}
\title{Sub-millisecond accurate timing of expression evaluation.}
\usage{
microbenchmark(
  ...,
  list = NULL,
  times = 100L,
  unit = NULL,
  check = NULL,
  control = list(),
  setup = NULL
)
}
\arguments{
\item{...}{Expressions to benchmark.}

\item{list}{List of unevaluated expressions to benchmark.}

\item{times}{Number of times to evaluate each expression.}

\item{unit}{Default unit used in \code{summary} and \code{print}.}

\item{check}{A function to check if the expressions are equal. By default \code{NULL} which omits the check.
In addition to a function, a string can be supplied.
The string \sQuote{equal} will compare all values using \code{\link{all.equal}}, \sQuote{equivalent} will compare all values using \code{\link{all.equal}} and check.attributes = FALSE, and \sQuote{identical} will compare all values using \code{\link{identical}}.}

\item{control}{List of control arguments. See Details.}

\item{setup}{An unevaluated expression to be run (untimed) before each benchmark expression.}
}
\value{
Object of class \sQuote{microbenchmark}, a data frame with
columns \code{expr} and \code{time}. \code{expr} contains the
deparsed expression as passed to \code{microbenchmark} or the name
of the argument if the expression was passed as a named
argument. \code{time} is the measured execution time of the
expression in nanoseconds. The order of the observations in the
data frame is the order in which they were executed.
}
\description{
\code{microbenchmark} serves as a more accurate replacement of the
often seen \code{system.time(replicate(1000, expr))}
expression. It tries hard to accurately measure only the time it
takes to evaluate \code{expr}. To achieved this, the
sub-millisecond (supposedly nanosecond) accurate timing functions
most modern operating systems provide are used. Additionally all
evaluations of the expressions are done in C code to minimize any
overhead.
}
\details{
This function is only meant for micro-benchmarking small pieces of
source code and to compare their relative performance
characteristics. You should generally avoid benchmarking larger
chunks of your code using this function. Instead, try using the R
profiler to detect hot spots and consider rewriting them in C/C++
or FORTRAN.

The \code{control} list can contain the following entries:
\describe{
\item{order}{the order in which the expressions are evaluated.
  \dQuote{random} (the default) randomizes the execution order,
  \dQuote{inorder} executes each expression in order and
  \dQuote{block} executes all repetitions of each expression
    as one block.}
\item{warmup}{the number of iterations to run the timing code before
  evaluating the expressions in \dots. These warm-up iterations are used
  to estimate the timing overhead as well as spinning up the processor
  from any sleep or idle states it might be in. The default value is 2.}
}
}
\note{
Depending on the underlying operating system, different
methods are used for timing. On Windows the
\code{QueryPerformanceCounter} interface is used to measure the
time passed. For Linux the \code{clock_gettime} API is used and on
Solaris the \code{gethrtime} function. Finally on MacOS X the,
undocumented, \code{mach_absolute_time} function is used to avoid
a dependency on the CoreServices Framework.

Before evaluating each expression \code{times} times, the overhead
of calling the timing functions and the C function call overhead
are estimated. This estimated overhead is subtracted from each
measured evaluation time. Should the resulting timing be negative,
a warning is thrown and the respective value is replaced by
\code{0}. If the timing is zero, a warning is raised.
Should all evaluations result in one of the two error conditions described above, an error is raised.

One platform on which the clock resolution is known to be too low to measure short runtimes with the required precision is 
Oracle\if{html}{\out{&reg;}}\if{latex}{\out{\textregistered\ }}\if{text}{(R)}
Solaris 
on some 
SPARC\if{html}{\out{&reg;}}\if{latex}{\out{\textregistered\ }}\if{text}{(R)}
hardware.
Reports of other platforms with similar problems are welcome.
Please contact the package maintainer.
}
\examples{
## Measure the time it takes to dispatch a simple function call
## compared to simply evaluating the constant \code{NULL}
f <- function() NULL
res <- microbenchmark(NULL, f(), times=1000L)

## Print results:
print(res)

## Plot results:
boxplot(res)

## Pretty plot:
if (requireNamespace("ggplot2")) {
  ggplot2::autoplot(res)
}

## Example check usage
my_check <- function(values) {
  all(sapply(values[-1], function(x) identical(values[[1]], x)))
}

f <- function(a, b)
  2 + 2

a <- 2
## Check passes
microbenchmark(2 + 2, 2 + a, f(2, a), f(2, 2), check=my_check)
\dontrun{
a <- 3
## Check fails
microbenchmark(2 + 2, 2 + a, f(2, a), f(2, 2), check=my_check)
}
## Example setup usage
set.seed(21)
x <- rnorm(10)
microbenchmark(x, rnorm(10), check=my_check, setup=set.seed(21))
## Will fail without setup
\dontrun{
microbenchmark(x, rnorm(10), check=my_check)
}
## using check
a <- 2
microbenchmark(2 + 2, 2 + a, sum(2, a), sum(2, 2), check='identical')
microbenchmark(2 + 2, 2 + a, sum(2, a), sum(2, 2), check='equal')
attr(a, 'abc') <- 123
microbenchmark(2 + 2, 2 + a, sum(2, a), sum(2, 2), check='equivalent')
## check='equal' will fail due to difference in attribute
\dontrun{
microbenchmark(2 + 2, 2 + a, sum(2, a), sum(2, 2), check='equal')
}
}
\seealso{
\code{\link{print.microbenchmark}} to display and
\code{\link{boxplot.microbenchmark}} or
\code{\link{autoplot.microbenchmark}} to plot the results.
}
\author{
Olaf Mersmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxplot.R
\name{boxplot.microbenchmark}
\alias{boxplot.microbenchmark}
\title{Boxplot of \code{microbenchmark} timings.}
\usage{
\method{boxplot}{microbenchmark}(
  x,
  unit = "t",
  log = TRUE,
  xlab,
  ylab,
  horizontal = FALSE,
  main = "microbenchmark timings",
  ...
)
}
\arguments{
\item{x}{A \code{microbenchmark} object.}

\item{unit}{Unit in which the results be plotted.}

\item{log}{Should times be plotted on log scale?}

\item{xlab}{X axis label.}

\item{ylab}{Y axis label.}

\item{horizontal}{Switch X and Y axes.}

\item{main}{Plot title.}

\item{...}{Passed on to boxplot.formula.}
}
\description{
Boxplot of \code{microbenchmark} timings.
}
\author{
Olaf Mersmann
}
